﻿using CommunityToolkit.Mvvm.Input;
using DynamicMVVMTreeViewHierarchical;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Media;

namespace MVVMTreeViewHierarchical
{
    public class MainWindowViewModel : CommunityToolkit.Mvvm.ComponentModel.ObservableObject
    {
        private int globalIndex = 0;
        private ObservableCollection<Parent> hierarchicalTestList = new ObservableCollection<Parent>();
        Random random = new Random();
        public RelayCommand<TreeView> LoadedCommand { get; set; }

        public ObservableCollection<Parent> HierarchicalTestList
        {
            get => hierarchicalTestList;
            set => SetProperty(ref hierarchicalTestList, value);
        }

        private int itemIndex;

        public int ItemIndex
        {
            get => itemIndex;
            set => SetProperty(ref itemIndex, value);
        }

        public MainWindowViewModel()
        {
            LoadedCommand = new RelayCommand<TreeView>(OnLoaded);   
        }

        private void OnLoaded(TreeView treeView)
        {
            Parent parent1 = new Parent();
            parent1.ParentName = "父节点1";
            parent1.ItemIndex = globalIndex++.ToString();
            for (int i = 0; i < 1000; i++)
            {
                Child child = new Child();
                child.Item1 = GetTestStr();
                child.Item2 = GetTestStr();
                child.ItemIndex = globalIndex++.ToString();
                parent1.ChildList.Add(child);
            }

            hierarchicalTestList.Add(parent1);

            Parent parent2 = new Parent();
            parent2.ParentName = "父节点2";
            parent2.ItemIndex = globalIndex++.ToString();
            for (int i = 0; i < 1000; i++)
            {
                Child child = new Child();
                child.Item1 = GetTestStr();
                child.Item2 = GetTestStr();
                child.ItemIndex = globalIndex++.ToString();
                parent2.ChildList.Add(child);
            }
            hierarchicalTestList.Add(parent2);

            Parent parent3 = new Parent();
            parent3.ParentName = "父节点3";
            parent3.ItemIndex = globalIndex++.ToString();
            for (int i = 0; i < 1000; i++)
            {
                Child child = new Child();
                child.Item1 = GetTestStr();
                child.Item2 = GetTestStr();
                child.ItemIndex = globalIndex++.ToString();
                parent3.ChildList.Add(child);
            }

            hierarchicalTestList.Add(parent3);

            //使用模板选择器
            treeView.ItemTemplateSelector = new NodeDataTemplateSelector();

            //使用代码创建模板
            //treeView.ItemTemplate = GetHierarchicalDataTemplate();

            treeView.ItemsSource = hierarchicalTestList;
        }

        private string GetTestStr()
        {

            var index = random.Next(0, DynamicMVVMTreeViewHierarchicalVariousData.Properties.Resources.String1.Length - 10);

            if (index >= DynamicMVVMTreeViewHierarchicalVariousData.Properties.Resources.String1.Length)
                return "TestString";


            return DynamicMVVMTreeViewHierarchicalVariousData.Properties.Resources.String1.Substring(index, 10);
        }

        private HierarchicalDataTemplate GetHierarchicalDataTemplate()
        {
            var parentHierarchicalDataTemplate = CreateParentHierarchicalDataTemplate();
            var childHierarchicalDataTemplate = CreateChildHierarchicalDataTemplate();
            parentHierarchicalDataTemplate.ItemTemplate = childHierarchicalDataTemplate;
            return parentHierarchicalDataTemplate;
        }

        private HierarchicalDataTemplate CreateParentHierarchicalDataTemplate()
        {          
            HierarchicalDataTemplate hierarchicalDataTemplate = new HierarchicalDataTemplate();
            hierarchicalDataTemplate.ItemsSource = new Binding("ChildList");
            hierarchicalDataTemplate.DataType = typeof(Parent);
            FrameworkElementFactory grid = new FrameworkElementFactory(typeof(Grid));

            FrameworkElementFactory columnDefinition1 = new FrameworkElementFactory(typeof(ColumnDefinition));
            columnDefinition1.SetValue(ColumnDefinition.WidthProperty, new GridLength(60));
            FrameworkElementFactory columnDefinition2 = new FrameworkElementFactory(typeof(ColumnDefinition));
            grid.AppendChild(columnDefinition1);
            grid.AppendChild(columnDefinition2);

            var labelFactory = new FrameworkElementFactory(typeof(Label));
            labelFactory.SetBinding(Label.ContentProperty, new Binding("ItemIndex"));
            var label2Factory = new FrameworkElementFactory(typeof(Label));
            label2Factory.SetBinding(Label.ContentProperty, new Binding("ParentName"));
            label2Factory.SetValue(Label.HorizontalContentAlignmentProperty, HorizontalAlignment.Left);
            label2Factory.SetValue(Grid.ColumnProperty, 1);
            grid.AppendChild(labelFactory);
            grid.AppendChild(label2Factory);
            hierarchicalDataTemplate.VisualTree = grid;
            return hierarchicalDataTemplate;            
        }

        private HierarchicalDataTemplate CreateChildHierarchicalDataTemplate()
        {
            HierarchicalDataTemplate hierarchicalDataTemplate = new HierarchicalDataTemplate();
            hierarchicalDataTemplate.DataType = typeof(Child);

            FrameworkElementFactory grid = new FrameworkElementFactory(typeof(Grid));
            FrameworkElementFactory columnDefinition1 = new FrameworkElementFactory(typeof(ColumnDefinition));
            columnDefinition1.SetValue(ColumnDefinition.WidthProperty, new GridLength(80));
            FrameworkElementFactory columnDefinition2 = new FrameworkElementFactory(typeof(ColumnDefinition));
            grid.AppendChild(columnDefinition1);
            grid.AppendChild(columnDefinition2);

            var labelFactory = new FrameworkElementFactory(typeof(Label));
            labelFactory.SetBinding(Label.ContentProperty, new Binding("ItemIndex"));
            grid.AppendChild(labelFactory);

            FrameworkElementFactory tb = new FrameworkElementFactory(typeof(WrapPanel));
            tb.SetValue(Grid.ColumnProperty, 1);
      
            var label2Factory = new FrameworkElementFactory(typeof(Label));
            label2Factory.SetBinding(Label.ContentProperty, new Binding("Item1"));
            label2Factory.SetValue(Label.HorizontalContentAlignmentProperty, HorizontalAlignment.Left);
            var label3Factory = new FrameworkElementFactory(typeof(Label));
            label3Factory.SetBinding(Label.ContentProperty, new Binding("Item2"));
            label3Factory.SetValue(Label.HorizontalContentAlignmentProperty, HorizontalAlignment.Left);
            label3Factory.SetValue(Label.MarginProperty, new Thickness(5, 0, 0, 0));
            label3Factory.SetValue(Label.ForegroundProperty, Brushes.Green);
            tb.AppendChild(label2Factory);
            tb.AppendChild(label3Factory);

            grid.AppendChild(tb);
            hierarchicalDataTemplate.VisualTree = grid;
            return hierarchicalDataTemplate;
        }
    }
}
